
; ********************************************************************
; calc.asm
 #include <p16F84A.inc>
; ********************************************************************

 UDATA

; for division routines 
AARGB1	   res 1
AARGB0 	   res 1
AARGB5     res 1
BARGB0	   res 1
BARGB1	   res 1
REMB1	   res 1
REMB0	   res 1
LOOPCOUNT  res 1

; for calculation routines
SLOP_LO	   res 1	
SLOP_HI	   res 1
TEMP	   res 1

TempC	   res 1	; final temperature value in centigrade
TempP	   res 1
Neg	   res 1	; negative value 

 global	TempCalc
 global	FXD1616U

 global	   TempC,TempP,Neg	

 extern	   WDTCOUNT_HI,WDTCOUNT_LO,TEMP_HI,TEMP_LO
 extern	   WDT_H1,WDT_H0,WDT_L1,WDT_L0	   

 CODE

; ********************************************************************
; Calculation for temperature in Centigrades
; input requires for the routine
; WDT Timeout count(16-bit)	- WDTCOUNT_HI,WDTCOUNT_LO 

; Calibration TEMP1		- TEMP_HI
; Calibration TEMP2		- TEMP_LO
; Corresponding COUNT(16-bit)
; for TEMP1			- WDT_H1,WDT_H0
; for TEMP2			- WDT_L1,WDT_L0
; output (Final Temperature)	- TempC 
; if negative     		- Neg = 01
; if positive 	  		- Neg = 00
; ********************************************************************

TempCalc
; calculation for SLOPE from two known temperature 
; Program based on (high Temperature) > (Low Temperature)

; M 	= (Y2-Y1)/(X2-X1)
; X1	->TEMP_LO->0x04		- LOW calibration temperature set by user
; X2	->TEMP_HI->0x00		- HIGH Calibration temperature set by user
; Y1	->WDT_L0 ->0x05 	- 16-bit WDT count at the time of 
;	->WDT_L1 ->0x06		- LOW temperature set
; Y2	->WDT_H0 ->0x02		- 16-bit WDT count at the time of 
;	->WDT_H1 ->0x01		- HIGH temperature set
; X1 	= X2- [(Y2-Y1)/M] 	- actual value of the temperature

NEW_VALUE			
	clrwdt	
	movf	TEMP_LO,w
	subwf	TEMP_HI,w	;[(TEMP_HI)-(TEMP_LO)] 
	movwf	BARGB1
	clrf	BARGB0

; [<WDT_H1:WDT_H0>-<WDT_L1-WDT_L0>]

	movf	WDT_L0,w	; (X2-X1)- 16 bit subtraction
	subwf	WDT_H0,w
	movwf	AARGB1		; lower 8-bit result of 16-bit subtraction
	btfss	STATUS,C	; check for carry
	incf	WDT_L1,f			
	movf	WDT_L1,w
	subwf	WDT_H1,w

high_val
	movwf	AARGB0		; upper 8 bit result of 16-bit subtraction

; <AARGB0:AARGB1> / BARGB0 = <AARGB0:AARGB1>

	call	FXD1616U 	; 16/16 division 
	movf	AARGB1,w
	movwf	BARGB1		; 16-bit SLOPE	
	movwf	SLOP_LO		; lower 8-bit of SLOPE
	movf	AARGB0,w
	movwf	BARGB0		; upper 8-bit of SLOPE
	movwf	SLOP_HI

; ********************************************************************
; X1=X2-[(Y2-Y1)/SLOPE]
; Y2 = <WDT_H1:WDT_H0>
; Y1 = <WDTCOUNT_HI:WDTCOUNT_LO>
; X2 = TEMP_HI
; X1 = new temperature(current temp.)
; ********************************************************************
	movf	WDTCOUNT_HI,w	; 16-bit subtraction
	subwf	WDT_H1,W	
	btfss	STATUS,C
	goto	CHANGEseq
	movf	WDTCOUNT_LO,w
	subwf	WDT_H0,w
	movwf	AARGB1		; low 8 bit
	btfss	STATUS,C	; check for carry
	incf	WDTCOUNT_HI,f	
	movf	WDTCOUNT_HI,w
	subwf	WDT_H1,w

high_val1
	movwf	AARGB0		; upper 8 bit

; <AARGB0:AARGB1> / <BARGB0:BARGB1> = <AARGB0:AARGB1>
;AARG, REM  <--  AARG / BARG

	call	FXD1616U 	; 16/16 division
	movf	AARGB1,w	; 
	movwf	TEMP		; 
; ********************************************************************
; X1 = X2-[(Y2-Y1)/SLOPE]
; X2 = TEMP_HI

	btfsc	Neg,0		
	goto	ADDvalue
	subwf	TEMP_HI,w
	btfss	STATUS,C
	goto	ChangeSign
PosSign
	movwf	TempC		; final value of temperature in celcius	
	movwf	TempP
	bcf	Neg,0
	return		
ChangeSign			; for negative value of the 
	movf	TEMP_HI,w	; temp.
	subwf	TEMP,w
	movwf	TempC	
	movwf	TempP
	bsf	Neg,0		; for negative value
;	bsf	Neg,2		; for -ve 'C' value
 	return
CHANGEseq
	bsf	Neg,0		;1	
	movf	WDT_H0,w
	subwf	WDTCOUNT_LO,w

	movwf	AARGB1
	movf	WDT_H1,w
	btfss	STATUS,C
	incf	WDT_H1,w
	subwf	WDTCOUNT_HI,w
	goto	high_val1

ADDvalue
	addwf	TEMP_HI,w
	goto	PosSign


; ********************************************************************
; Division routine 16/16
UDIV1616L       macro
;       restore = 23 clks,  nonrestore = 17 clks
;       Max Timing:     2+15*23+22 = 369 clks
;       Min Timing:     2+15*17+16 = 273 clks
;       PM: 24                                  DM: 7
                MOVLW           D'16'
                MOVWF           LOOPCOUNT
LOOPU1616       RLF             AARGB0,W
                RLF             REMB1, F
                RLF             REMB0, F
                MOVF            BARGB1,W
                SUBWF           REMB1, F
                MOVF            BARGB0,W
                BTFSS           STATUS,C
                INCFSZ          BARGB0,W
                SUBWF           REMB0, F
                BTFSC           STATUS,C
                GOTO            UOK66LL         
                MOVF            BARGB1,W
                ADDWF           REMB1, F
                MOVF            BARGB0,W
                BTFSC           STATUS,C
                INCFSZ          BARGB0,W
                ADDWF           REMB0, F
                BCF             STATUS,C
UOK66LL 	RLF             AARGB1, F
                RLF             AARGB0, F
                DECFSZ          LOOPCOUNT, F
                GOTO            LOOPU1616
                endm
; ********************************************************************
;       16/16 Bit Unsigned Fixed Point Divide 16/16 -> 16.16
;       Input:  16 bit unsigned fixed point dividend in AARGB0, AARGB1
;               16 bit unsigned fixed point divisor in BARGB0, BARGB1
;       Use:    CALL    FXD1616U
;       Output: 16 bit unsigned fixed point quotient in AARGB0, AARGB1
;               16 bit unsigned fixed point remainder in REMB0, REMB1
;       Result: AARG, REM  <--  AARG / BARG
;       Max Timing:     2+369+2 = 373 clks
;       Min Timing:     2+273+2 = 277 clks
;       PM: 2+24+1 = 27         DM: 7
; <AARGB0:AARGB1> / <BARGB0:BARGB1> = <AARGB0:AARGB1>
; remainder in <REMB0:REMB1>

FXD1616U        CLRF            REMB0
                CLRF            REMB1
                UDIV1616L
                RETLW           0x00
; ********************************************************************
 
 end


